/*****************************************************************************/
/* Copyright (C) 2003-2007, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

// ClassType_H.template
#ifndef _97MP7JRSAUDIOHARMONICITYTYPE_H
#define _97MP7JRSAUDIOHARMONICITYTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsAudioHarmonicityTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtInclude.h


#include "Mp7JrsAudioDType.h"
#include "Mp7JrsDType.h"
#include "Mp7JrsMpeg7BaseType.h"
class IMp7JrsAudioDType;
class IMp7JrsAudioHarmonicityType;
class IMp7JrsAudioLLDScalarType;
typedef Mp7JrsPtr< IMp7JrsAudioDType > Mp7JrsAudioDPtr;
typedef Mp7JrsPtr< IMp7JrsAudioHarmonicityType> Mp7JrsAudioHarmonicityPtr;
typedef Mp7JrsPtr< IMp7JrsAudioLLDScalarType > Mp7JrsAudioLLDScalarPtr;

/** 
 * Generated interface IMp7JrsAudioHarmonicityType for class Mp7JrsAudioHarmonicityType<br>
 * Located at: audio-2001.xsd, line 159<br>
 * Classified: Class<br>
 * Derived from: AudioDType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsAudioHarmonicityType 
 :
        public IMp7JrsAudioDType
{
public:
    // TODO: make these protected?
    IMp7JrsAudioHarmonicityType();
    virtual ~IMp7JrsAudioHarmonicityType();
        /** @name Sequence

         */
        //@{
    /**
     * Get HarmonicRatio element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetHarmonicRatio() const = 0;

    /**
     * Set HarmonicRatio element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    // Mandatory abstract element, possible types are
    // Mp7JrsAudioFundamentalFrequencyPtr
    // Mp7JrsAudioPowerPtr
    // Mp7JrsAudioSpectrumCentroidPtr
    // Mp7JrsAudioSpectrumSpreadPtr
    // Mp7JrsAudioWaveformPtr
    // Mp7JrsHarmonicSpectralCentroidPtr
    // Mp7JrsHarmonicSpectralDeviationPtr
    // Mp7JrsHarmonicSpectralSpreadPtr
    // Mp7JrsHarmonicSpectralVariationPtr
    // Mp7JrsLogAttackTimePtr
    // Mp7JrsSpectralCentroidPtr
    // Mp7JrsTemporalCentroidPtr
    virtual void SetHarmonicRatio(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get UpperLimitOfHarmonicity element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetUpperLimitOfHarmonicity() const = 0;

    /**
     * Set UpperLimitOfHarmonicity element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    // Mandatory abstract element, possible types are
    // Mp7JrsAudioFundamentalFrequencyPtr
    // Mp7JrsAudioPowerPtr
    // Mp7JrsAudioSpectrumCentroidPtr
    // Mp7JrsAudioSpectrumSpreadPtr
    // Mp7JrsAudioWaveformPtr
    // Mp7JrsHarmonicSpectralCentroidPtr
    // Mp7JrsHarmonicSpectralDeviationPtr
    // Mp7JrsHarmonicSpectralSpreadPtr
    // Mp7JrsHarmonicSpectralVariationPtr
    // Mp7JrsLogAttackTimePtr
    // Mp7JrsSpectralCentroidPtr
    // Mp7JrsTemporalCentroidPtr
    virtual void SetUpperLimitOfHarmonicity(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of AudioHarmonicityType.
     * Currently this type contains 2 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>HarmonicRatio</li>
     * <li>UpperLimitOfHarmonicity</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsAudioHarmonicityType<br>
 * Located at: audio-2001.xsd, line 159<br>
 * Classified: Class<br>
 * Derived from: AudioDType (Class)<br>
 * Defined in audio-2001.xsd, line 159.<br>
 */
class MP7JRS_EXPORT Mp7JrsAudioHarmonicityType :
        public IMp7JrsAudioHarmonicityType
{
    friend class Mp7JrsAudioHarmonicityTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsAudioHarmonicityType
     
     * @return Mp7JrsPtr< Mp7JrsAudioDType >
     */
    virtual Mp7JrsPtr< Mp7JrsAudioDType > GetBase() const;
        /* @name Sequence

         */
        //@{
    /*
     * Get HarmonicRatio element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetHarmonicRatio() const;

    /*
     * Set HarmonicRatio element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    // Mandatory abstract element, possible types are
    // Mp7JrsAudioFundamentalFrequencyPtr
    // Mp7JrsAudioPowerPtr
    // Mp7JrsAudioSpectrumCentroidPtr
    // Mp7JrsAudioSpectrumSpreadPtr
    // Mp7JrsAudioWaveformPtr
    // Mp7JrsHarmonicSpectralCentroidPtr
    // Mp7JrsHarmonicSpectralDeviationPtr
    // Mp7JrsHarmonicSpectralSpreadPtr
    // Mp7JrsHarmonicSpectralVariationPtr
    // Mp7JrsLogAttackTimePtr
    // Mp7JrsSpectralCentroidPtr
    // Mp7JrsTemporalCentroidPtr
    virtual void SetHarmonicRatio(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get UpperLimitOfHarmonicity element.
     * @return Mp7JrsPtr< Mp7JrsNode >
     * Possible types (within this library):
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    virtual Mp7JrsPtr< Mp7JrsNode > GetUpperLimitOfHarmonicity() const;

    /*
     * Set UpperLimitOfHarmonicity element.
     * @param item const Mp7JrsPtr< Mp7JrsNode > &
     * Valid types for item:
     * <ul>
     * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
     * <li>IMp7JrsAudioPowerPtr</li>
     * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
     * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
     * <li>IMp7JrsAudioWaveformPtr</li>
     * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
     * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
     * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
     * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
     * <li>IMp7JrsLogAttackTimePtr</li>
     * <li>IMp7JrsSpectralCentroidPtr</li>
     * <li>IMp7JrsTemporalCentroidPtr</li>
     * </ul>
     */
    // Mandatory abstract element, possible types are
    // Mp7JrsAudioFundamentalFrequencyPtr
    // Mp7JrsAudioPowerPtr
    // Mp7JrsAudioSpectrumCentroidPtr
    // Mp7JrsAudioSpectrumSpreadPtr
    // Mp7JrsAudioWaveformPtr
    // Mp7JrsHarmonicSpectralCentroidPtr
    // Mp7JrsHarmonicSpectralDeviationPtr
    // Mp7JrsHarmonicSpectralSpreadPtr
    // Mp7JrsHarmonicSpectralVariationPtr
    // Mp7JrsLogAttackTimePtr
    // Mp7JrsSpectralCentroidPtr
    // Mp7JrsTemporalCentroidPtr
    virtual void SetUpperLimitOfHarmonicity(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of AudioHarmonicityType.
     * Currently this type contains 2 child elements of type "Mp7JrsNodePtr".

     * <ul>
     * <li>HarmonicRatio</li>
     * <li>UpperLimitOfHarmonicity</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsAudioHarmonicityType();

protected:
    Mp7JrsAudioHarmonicityType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:

    // Base Class
    Mp7JrsPtr< Mp7JrsAudioDType > m_Base;

    Mp7JrsPtr< Mp7JrsNode > m_HarmonicRatio;
    Mp7JrsPtr< Mp7JrsNode > m_UpperLimitOfHarmonicity;

// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();

};

#endif // _97MP7JRSAUDIOHARMONICITYTYPE_H

