/*****************************************************************************/
/* Copyright (C) 2003-2007, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

// ClassType_H.template
#ifndef _1063MP7JRSINLINEMEDIATYPE_H
#define _1063MP7JRSINLINEMEDIATYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsInlineMediaTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtInclude.h


class IMp7JrsInlineMediaType;
class IMp7JrsmimeType;
typedef Mp7JrsPtr< IMp7JrsInlineMediaType> Mp7JrsInlineMediaPtr;
typedef Mp7JrsPtr< IMp7JrsmimeType > Mp7JrsmimePtr;

/** 
 * Generated interface IMp7JrsInlineMediaType for class Mp7JrsInlineMediaType<br>
 * Located at: mds-2001.xsd, line 1106<br>
 * Classified: Class<br>
 */
class MP7JRS_EXPORT IMp7JrsInlineMediaType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsInlineMediaType();
    virtual ~IMp7JrsInlineMediaType();
    /**
     * Get type attribute.
     * @return Mp7JrsmimePtr
     */
    virtual Mp7JrsmimePtr Gettype() const = 0;

    /**
     * Set type attribute.
     * @param item const Mp7JrsmimePtr &
     */
    // Mandatory
    virtual void Settype(const Mp7JrsmimePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        /** @name Choice

         */
        //@{
    /**
     * Get MediaData16 element.
     * @return XMLCh *
     */
    virtual XMLCh * GetMediaData16() const = 0;

    /**
     * Set MediaData16 element.
     * @param item XMLCh *
     */
    // Mandatory			
    virtual void SetMediaData16(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get MediaData64 element.
     * @return XMLCh *
     */
    virtual XMLCh * GetMediaData64() const = 0;

    /**
     * Set MediaData64 element.
     * @param item XMLCh *
     */
    // Mandatory			
    virtual void SetMediaData64(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
    /**
     * Gets or creates Mp7JrsNodePtr child elements of InlineMediaType.
     * Currently just supports rudimentary XPath expressions as InlineMediaType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsInlineMediaType<br>
 * Located at: mds-2001.xsd, line 1106<br>
 * Classified: Class<br>
 * Defined in mds-2001.xsd, line 1106.<br>
 */
class MP7JRS_EXPORT Mp7JrsInlineMediaType :
        public IMp7JrsInlineMediaType
{
    friend class Mp7JrsInlineMediaTypeFactory; // constructs objects

public:
    /*
     * Get type attribute.
     * @return Mp7JrsmimePtr
     */
    virtual Mp7JrsmimePtr Gettype() const;

    /*
     * Set type attribute.
     * @param item const Mp7JrsmimePtr &
     */
    // Mandatory
    virtual void Settype(const Mp7JrsmimePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        /* @name Choice

         */
        //@{
    /*
     * Get MediaData16 element.
     * @return XMLCh *
     */
    virtual XMLCh * GetMediaData16() const;

    /*
     * Set MediaData16 element.
     * @param item XMLCh *
     */
    // Mandatory			
    virtual void SetMediaData16(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get MediaData64 element.
     * @return XMLCh *
     */
    virtual XMLCh * GetMediaData64() const;

    /*
     * Set MediaData64 element.
     * @param item XMLCh *
     */
    // Mandatory			
    virtual void SetMediaData64(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
    /*
     * Gets or creates Mp7JrsNodePtr child elements of InlineMediaType.
     * Currently just supports rudimentary XPath expressions as InlineMediaType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsInlineMediaType();

protected:
    Mp7JrsInlineMediaType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrsmimePtr m_type;


// DefinionChoice
    XMLCh * m_MediaData16;
    XMLCh * m_MediaData64;
// End DefinionChoice

// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();

};

#endif // _1063MP7JRSINLINEMEDIATYPE_H

