/*****************************************************************************/
/* Copyright (C) 2003-2007, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

// ClassType_H.template
#ifndef _1115MP7JRSLEXICONTYPE_H
#define _1115MP7JRSLEXICONTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsLexiconTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsLexiconType_ExtInclude.h


#include "Mp7JrsHeaderType.h"
#include "Mp7JrsMpeg7BaseType.h"
class IMp7JrsHeaderType;
class IMp7JrsLexiconType;
typedef Mp7JrsPtr< IMp7JrsHeaderType > Mp7JrsHeaderPtr;
typedef Mp7JrsPtr< IMp7JrsLexiconType> Mp7JrsLexiconPtr;

/** 
 * Generated interface IMp7JrsLexiconType for class Mp7JrsLexiconType<br>
 * Located at: audio-2001.xsd, line 530<br>
 * Classified: Class<br>
 * Derived from: HeaderType (Class)<br>
 * Abstract class <br>
 */
class MP7JRS_EXPORT IMp7JrsLexiconType 
 :
        public IMp7JrsHeaderType
{
public:
    // TODO: make these protected?
    IMp7JrsLexiconType();
    virtual ~IMp7JrsLexiconType();
    /**
     * Get numOfOriginalEntries attribute.
     * @return unsigned
     */
    virtual unsigned GetnumOfOriginalEntries() const = 0;

    /**
     * Get numOfOriginalEntries attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistnumOfOriginalEntries() const = 0;

    /**
     * Set numOfOriginalEntries attribute.
     * @param item unsigned
     */
    virtual void SetnumOfOriginalEntries(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate numOfOriginalEntries attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatenumOfOriginalEntries() = 0;

    /**
     * @if Show_All_Abstract_Class_Members
     * Get id attribute.
<br>
     * (Inherited from Mp7JrsHeaderType)
     * @return XMLCh *
     * @endif
     */
    virtual XMLCh * Getid() const = 0;

    /**
     * @if Show_All_Abstract_Class_Members
     * Get id attribute validity information.
     * (Inherited from Mp7JrsHeaderType)
     * @return true if attribute value is valid, false if not.
     * @endif
     */
    virtual bool Existid() const = 0;

    /**
     * @if Show_All_Abstract_Class_Members
     * Set id attribute.
<br>
     * (Inherited from Mp7JrsHeaderType)
     * @param item XMLCh *
     * @endif
     */
    virtual void Setid(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * @if Show_All_Abstract_Class_Members
     * Invalidate id attribute.
     * (Inherited from Mp7JrsHeaderType)
     
     * (Validate it again by setting a value)
     * @endif
     */
    virtual void Invalidateid() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of LexiconType.
     * Currently just supports rudimentary XPath expressions as LexiconType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsLexiconType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsLexiconType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsLexiconType<br>
 * Located at: audio-2001.xsd, line 530<br>
 * Classified: Class<br>
 * Derived from: HeaderType (Class)<br>
 * Abstract class <br>
 * Defined in audio-2001.xsd, line 530.<br>
 */
class MP7JRS_EXPORT Mp7JrsLexiconType :
        public IMp7JrsLexiconType
{
    friend class Mp7JrsLexiconTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsLexiconType
     
     * @return Mp7JrsPtr< Mp7JrsHeaderType >
     */
    virtual Mp7JrsPtr< Mp7JrsHeaderType > GetBase() const;
    /*
     * Get numOfOriginalEntries attribute.
     * @return unsigned
     */
    virtual unsigned GetnumOfOriginalEntries() const;

    /*
     * Get numOfOriginalEntries attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistnumOfOriginalEntries() const;

    /*
     * Set numOfOriginalEntries attribute.
     * @param item unsigned
     */
    virtual void SetnumOfOriginalEntries(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate numOfOriginalEntries attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatenumOfOriginalEntries();

    /*
     * Get id attribute.
<br>
     * (Inherited from Mp7JrsHeaderType)
     * @return XMLCh *
     */
    virtual XMLCh * Getid() const;

    /*
     * Get id attribute validity information.
     * (Inherited from Mp7JrsHeaderType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existid() const;

    /*
     * Set id attribute.
<br>
     * (Inherited from Mp7JrsHeaderType)
     * @param item XMLCh *
     */
    virtual void Setid(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate id attribute.
     * (Inherited from Mp7JrsHeaderType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateid();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of LexiconType.
     * Currently just supports rudimentary XPath expressions as LexiconType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsLexiconType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsLexiconType();

protected:
    Mp7JrsLexiconType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    unsigned m_numOfOriginalEntries;
    bool m_numOfOriginalEntries_Exist;

    // Base Class
    Mp7JrsPtr< Mp7JrsHeaderType > m_Base;


// no includefile for extension defined 
// file Mp7JrsLexiconType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();

};

#endif // _1115MP7JRSLEXICONTYPE_H

