/*****************************************************************************/
/* Copyright (C) 2003-2007, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/

#ifndef MP7JRS_SCHEMALOCATIONMAP_H
#define MP7JRS_SCHEMALOCATIONMAP_H

#include "Mp7JrsDefines.h"
#include <xercesc/util/XMLString.hpp>
#include "Mp7JrsKeyValuePair.h" // To support Xerces 2.2.0 AND higher!
#include "Mp7JrsRefVectorOf.h"

typedef struct {
	Mp7JrsRefVectorOf<XMLCh>* prefixes;
	XMLCh* location;
	int prefix_it;
} SchemaProperties;

typedef Mp7JrsKeyValuePair<XMLCh*,SchemaProperties*> SchemaData;

class MP7JRS_EXPORT Mp7JrsSchemaLocationMap
{

public: 
	/** constructs location map */
	Mp7JrsSchemaLocationMap(void);
	~Mp7JrsSchemaLocationMap(void);

	/** Sets location for specified schema. If there is an existing entry for this schema, it will be overwritten. 
	  * Otherwise a new entry will be created. The strings passed as arguments will be copied.
	  * @param schemaName Namespace
	  * @param prefix default namespace prefix (including xmlns:)
	  * @param schemaLocation URN of the schema file. If this is an empty string, no entry will be added to the
	  * string generated by ToText.
	  */
	void SetSchemaLocation(const XMLCh* schemaName, const XMLCh* prefix, const XMLCh* schemaLocation);

	/** Adds a prefix alias for a namespace already defined. If the namespace is not already defined, it
	  * will NOT be created.
	  * @param schemaName Name of the schema namespace.
	  * @param prefix The prefix alias for this namespace (including xmlns:)
	  */
	void AddPrefixAlias(XMLCh* schemaName, XMLCh* prefix);

	/** Sets location for specified schema. If there is an existing entry for this schema, it will be overwritten. 
	  * Otherwise a new entry will be created. The strings passed as arguments will be copied.
	  * @param schemaName Namespace
	  * @param prefix default namespace prefix (including xmlns:)
	  * @param schemaLocation URN of the schema file. If this is an empty string, no entry will be added to the
	  * string generated by ToText.
	  */
	void SetSchemaLocation(const char* schemaName, const char* prefix, const char* schemaLocation);

	/** Adds a prefix alias for a namespace already defined. If the namespace is not already defined, it
	  * will NOT be created.
	  * @param schemaName Name of the schema namespace.
	  * @param prefix The prefix alias for this namespace (including xmlns:)
	  */
	void AddPrefixAlias(const char* schemaName, const char* prefix);

	/** Gets the location of the specified schema.
	  * @param schemaName Namespace
	  * @return The URN of the schema file. Returns NULL if no location for this schema has been found.
	  */
	const XMLCh* GetSchemaLocation(const XMLCh* schemaName);

	/** Gets the default namespace prefix of the specified schema.
	  * @param schemaName Namespace
	  * @return The default prefix of the namespace. Returns NULL if no prefix for this schema has been found.
	  */
	const XMLCh* GetFirstSchemaPrefix(const XMLCh* schemaName);

	/** Gets the next namespace prefix of the specified schema.
	  * @param schemaName Namespace
	  * @return The next prefix of the namespace. Returns NULL if no more prefixes for this schema has been found.
	  */
	const XMLCh* GetNextSchemaPrefix(const XMLCh* schemaName);

	/** Gets the first namespace's name and prefix.
	  * @return true if values set in the arguments, false if no namespace has been found.
	  */
	bool GetFirstNamespace(const XMLCh* * prefix, const XMLCh* * ns);

	/** Gets the next namespace's name and prefix.
	  * @return true if values set in the arguments, false if no more namespace has been found.
	  */
	bool GetNextNamespace(const XMLCh* * prefix, const XMLCh* * ns);

	/** Returns the content of the schema location map as space separated pairs of namespace and schema locations 
	  * It is the responsibility of the caller to deallocate the string.
	  */
	XMLCh* ToText() const;

private:
	Mp7JrsRefVectorOf<SchemaData>*	_locationMap;

	unsigned int _nsIter;
	bool _newNs;

	/** Get key value pair for specified schema */
	SchemaData* _GetSchemaData(const XMLCh* schemaName);

	/** Deletes a schema properties entry */
	void _DeleteSchemaProperties(SchemaProperties** sp);

};
#endif /* MP7JRS_SCHEMALOCATIONMAP_H */
