/* CreateDocument MPEG-7 JRS sample program.
 * 
 * (c) 2003-2007 by JOANNEUM RESEARCH, Graz, Austria.
 */


#include <iostream>
#include <cstdio>
#include <xercesc/util/PlatformUtils.hpp>

#ifdef __APPLE__

#include "Mp7Jrs/Mp7JrsArchive.h"

#include "Mp7Jrs/Mp7JrsException.h"
#include "Mp7Jrs/Mp7JrsFactory.h"
#include "Mp7Jrs/Mp7JrsClass.h"
#include "Mp7Jrs/Mp7JrsMpeg7_LocalType.h"
#include "Mp7Jrs/Mp7JrsMpeg7_Description_CollectionType.h"
#include "Mp7Jrs/Mp7JrsContentEntityType.h"
#include "Mp7Jrs/Mp7JrsContentEntityType_MultimediaContent_CollectionType.h"
#include "Mp7Jrs/Mp7JrsImageType.h"
#include "Mp7Jrs/Mp7JrsMediaLocatorType.h"
#include "Mp7Jrs/Mp7JrsStillRegionType.h"
#include "Mp7Jrs/Mp7JrsImageTextType.h"
#include "Mp7Jrs/Mp7JrsCreationType.h"
#include "Mp7Jrs/Mp7JrsCreationInformationType.h"
#include "Mp7Jrs/Mp7JrsCreationType_Title_CollectionType.h"
#include "Mp7Jrs/Mp7JrsTitleType.h"
#include "Mp7Jrs/Mp7JrsTextAnnotationType.h"
#include "Mp7Jrs/Mp7JrsTextAnnotationType_LocalType.h"
#include "Mp7Jrs/Mp7JrsTextualType.h"
#include "Mp7Jrs/Mp7JrsTextAnnotationType_CollectionType.h"
#include "Mp7Jrs/Mp7JrsSegmentType_TextAnnotation_CollectionType.h"
#include "Mp7Jrs/Mp7JrsStillRegionType_CollectionType.h"
#include "Mp7Jrs/Mp7JrsStillRegionType_LocalType.h"
#include "Mp7Jrs/Mp7JrsScalableColorType.h"
#include "Mp7Jrs/Mp7JrsintegerVector.h"
#include "Mp7Jrs/Mp7JrsSegmentType_CollectionType.h"
#include "Mp7Jrs/Mp7JrsSegmentType_LocalType.h"
#include "Mp7Jrs/Mp7JrsSemanticType.h"
#include "Mp7Jrs/Mp7JrsSemanticBaseType_Label_CollectionType.h"
#include "Mp7Jrs/Mp7JrsTermUseType.h"
#include "Mp7Jrs/Mp7JrsInlineTermDefinitionType_Name_CollectionType.h"
#include "Mp7Jrs/Mp7JrsInlineTermDefinitionType_Name_LocalType.h"
#include "Mp7Jrs/Mp7JrsSemanticBagType_CollectionType.h"
#include "Mp7Jrs/Mp7JrsSemanticBagType_LocalType.h"
#include "Mp7Jrs/Mp7JrsEventType.h"

#else

#include "Mp7JrsArchive.h"

#include "Mp7JrsException.h"
#include "Mp7JrsFactory.h"
#include "Mp7JrsClass.h"
#include "Mp7JrsMpeg7_LocalType.h"
#include "Mp7JrsMpeg7_Description_CollectionType.h"
#include "Mp7JrsContentEntityType.h"
#include "Mp7JrsContentEntityType_MultimediaContent_CollectionType.h"
#include "Mp7JrsImageType.h"
#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsStillRegionType.h"
#include "Mp7JrsImageTextType.h"
#include "Mp7JrsCreationType.h"
#include "Mp7JrsCreationInformationType.h"
#include "Mp7JrsCreationType_Title_CollectionType.h"
#include "Mp7JrsTitleType.h"
#include "Mp7JrsTextAnnotationType.h"
#include "Mp7JrsTextAnnotationType_LocalType.h"
#include "Mp7JrsTextualType.h"
#include "Mp7JrsTextAnnotationType_CollectionType.h"
#include "Mp7JrsSegmentType_TextAnnotation_CollectionType.h"
#include "Mp7JrsStillRegionType_CollectionType.h"
#include "Mp7JrsStillRegionType_LocalType.h"
#include "Mp7JrsScalableColorType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsSegmentType_CollectionType.h"
#include "Mp7JrsSegmentType_LocalType.h"
#include "Mp7JrsSemanticType.h"
#include "Mp7JrsSemanticBaseType_Label_CollectionType.h"
#include "Mp7JrsTermUseType.h"
#include "Mp7JrsInlineTermDefinitionType_Name_CollectionType.h"
#include "Mp7JrsInlineTermDefinitionType_Name_LocalType.h"
#include "Mp7JrsSemanticBagType_CollectionType.h"
#include "Mp7JrsSemanticBagType_LocalType.h"
#include "Mp7JrsEventType.h"


#endif

using namespace std;

Mp7JrsMediaLocatorPtr createMediaLocator(const char* loc) {
  Mp7JrsMediaLocatorPtr mediaLocator = CreateMediaLocatorType;
  mediaLocator->SetMediaUri(XMLString::transcode(loc));
  return mediaLocator;
}

Mp7JrsCreationInformationPtr createCreateInformation(const char* content) {
  Mp7JrsCreationInformationPtr ci = CreateCreationInformationType;
  Mp7JrsCreationPtr ct = CreateCreationType;
  Mp7JrsCreationType_Title_CollectionPtr ctColl = CreateCreationType_Title_CollectionType;
  Mp7JrsTitlePtr title = CreateTitleType;
  title->SetContent(XMLString::transcode(content));
  ctColl->addElement(title);
  ct->SetTitle(ctColl);
  ci->SetCreation(ct);
  return ci;
}

Mp7JrsSegmentType_TextAnnotation_CollectionPtr createTextAnnotation(const char* content) {
  Mp7JrsTextAnnotationType_CollectionPtr taColl = CreateTextAnnotationType_CollectionType;
  Mp7JrsTextAnnotationPtr ta = CreateTextAnnotationType;
  Mp7JrsTextAnnotationType_LocalPtr taLocal = CreateTextAnnotationType_LocalType;
  Mp7JrsTextualPtr text = CreateTextualType;
  text->SetContent(XMLString::transcode(content));
  taLocal->SetFreeTextAnnotation(text);
  taColl->addElement(taLocal);
  ta->SetTextAnnotationType_LocalType(taColl);
  Mp7JrsSegmentType_TextAnnotation_CollectionPtr sttaColl = CreateSegmentType_TextAnnotation_CollectionType;
  sttaColl->addElement(ta);
  return sttaColl;
}

Mp7JrsSegmentType_CollectionPtr createSemanticSection(const char* labelContent, const char* eventContent) {
  // semantic element
  Mp7JrsSegmentType_CollectionPtr segLocalColl = CreateSegmentType_CollectionType;
  Mp7JrsSegmentType_LocalPtr segLocal = CreateSegmentType_LocalType;
  Mp7JrsSemanticPtr semantic = CreateSemanticType;
  // label
  Mp7JrsSemanticBaseType_Label_CollectionPtr semanticLabelColl = CreateSemanticBaseType_Label_CollectionType;
  Mp7JrsTermUsePtr label = CreateTermUseType;
  Mp7JrsInlineTermDefinitionType_Name_CollectionPtr nameColl = CreateInlineTermDefinitionType_Name_CollectionType;
  Mp7JrsInlineTermDefinitionType_Name_LocalPtr name = CreateInlineTermDefinitionType_Name_LocalType;
  // local type etc. for semantic base
  Mp7JrsSemanticBagType_CollectionPtr sbLocalColl = CreateSemanticBagType_CollectionType;
  Mp7JrsSemanticBagType_LocalPtr sbLocal = CreateSemanticBagType_LocalType;
  Mp7JrsEventPtr event = CreateEventType;
  Mp7JrsSemanticBaseType_Label_CollectionPtr eventLabelColl = CreateSemanticBaseType_Label_CollectionType;
  Mp7JrsTermUsePtr eventLabel = CreateTermUseType;
  Mp7JrsInlineTermDefinitionType_Name_CollectionPtr eventNameColl = CreateInlineTermDefinitionType_Name_CollectionType;
  Mp7JrsInlineTermDefinitionType_Name_LocalPtr eventName = CreateInlineTermDefinitionType_Name_LocalType;
  name->SetContent(XMLString::transcode(labelContent));
  nameColl->addElement(name);
  label->SetName(nameColl);
  semanticLabelColl->addElement(label);
  semantic->SetLabel(semanticLabelColl);
  eventName->SetContent(XMLString::transcode(eventContent));
  eventNameColl->addElement(eventName);
  eventLabel->SetName(eventNameColl);
  eventLabelColl->addElement(eventLabel);
  event->SetLabel(eventLabelColl);
  sbLocal->SetSemanticBase(event);
  sbLocalColl->addElement(sbLocal);
  semantic->SetSemanticBagType_LocalType(sbLocalColl);
  segLocal->SetSemantic(semantic);
  segLocalColl->addElement(segLocal);
  return segLocalColl;
}

Mp7JrsStillRegionType_CollectionPtr createVisualDescriptor() {
  Mp7JrsStillRegionType_CollectionPtr srLocalColl = CreateStillRegionType_CollectionType;
  Mp7JrsStillRegionType_LocalPtr srLocal = CreateStillRegionType_LocalType;
  Mp7JrsScalableColorPtr color = CreateScalableColorType;
  color->SetnumOfCoeff(Mp7JrsScalableColorType_numOfCoeff_LocalType::_16);
  color->SetnumOfBitplanesDiscarded(Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::_0);
  Mp7JrsintegerVectorPtr intV = CreateintegerVector;
  int i = 0;
  while (i < 16) {
    i++;
    intV->addElement(i%10);
  }
  color->SetCoeff(intV);
  srLocal->SetVisualDescriptor(color);
  srLocalColl->addElement(srLocal);
  return srLocalColl;
}

Mp7JrsStillRegionPtr createStillRegion() {
  // create still region type (for image type => <image>)
  Mp7JrsStillRegionPtr stillRegion = CreateStillRegionType;
  // add media locator
  stillRegion->SetMediaLocator(createMediaLocator("image.jpg"));
  // add creation information
  stillRegion->SetCreationInformation(createCreateInformation("World Series Game 3"));
  // add text annotation
  stillRegion->SetTextAnnotation(createTextAnnotation("Game winning homerun"));
  // add semantic section
  stillRegion->SetSegmentType_LocalType(createSemanticSection("Last inning", "Game winning homerun"));
  // add visual descriptor
  stillRegion->SetStillRegionType_LocalType(createVisualDescriptor());
  return stillRegion;
}

Mp7JrsMpeg7_Description_CollectionPtr createDescription() {
  // description collection
  Mp7JrsMpeg7_Description_CollectionPtr descColl = CreateMpeg7_Description_CollectionType;
  // description of type content entity
  Mp7JrsContentEntityPtr contentEntity = CreateContentEntityType;
  // multimedia content collection
  Mp7JrsContentEntityType_MultimediaContent_CollectionPtr mmContentColl = CreateContentEntityType_MultimediaContent_CollectionType;
  // image type
  Mp7JrsImagePtr mmContent = CreateImageType;
  // add still region (<image> tag) to the image type 
  mmContent->SetImage(createStillRegion());
  // add image type to multimedia content collection
  mmContentColl->addElement(mmContent);
  // add the multimedia content collection to the content entity
  contentEntity->SetMultimediaContent(mmContentColl);
  // add the content entity to the description collection
  descColl->addElement(contentEntity);
  return descColl;
}

void rebuildTestTree(Mp7JrsMpeg7_LocalPtr root) {
  // add the description collection to the mpeg7 root node
  root->SetDescription(createDescription());
}

int main(int argc, char** argv) {

	XMLPlatformUtils::Initialize();
	Mp7JrsFactory::Initialize();

	// block for smart pointers
	try {
		Mp7JrsArchive archive;
		archive.SetEncoding("UTF-8");
		archive.SetDoValidation(false);
		Mp7JrsNodePtr root = archive.FromFile("test.xml");
		// access via description collection
		Mp7JrsMpeg7_Description_CollectionPtr dcp = ((Mp7JrsMpeg7_LocalPtr)root)->GetDescription();
		// delete all description elements (only root will be left)
		dcp->removeAllElements();
		// example: rebuild the tree manually
		rebuildTestTree(root);
		archive.ToFile("out.xml",root);
	}
	catch (Mp7JrsException e) {
		printf("Exception: %s\n",e.GetMessage());
	}

	Mp7JrsFactory::Terminate();
	XMLPlatformUtils::Terminate();

	printf("done.");
	getchar();
	return 0;
}
