/* Simple MPEG-7 JRS sample program.
 * 
 * (c) 2003-2007 by JOANNEUM RESEARCH, Graz, Austria.
 */

#include <stdio.h>

// Xerces utilities
#include <xercesc/util/PlatformUtils.hpp>

#ifdef __APPLE__
// archive class used for (de-) serializing
#include "Mp7Jrs/Mp7JrsArchive.h"

// factory registry and defines
#include "Mp7Jrs/Mp7JrsFactory.h"
#include "Mp7Jrs/Mp7JrsFactoryDefines.h"
#else
// archive class used for (de-) serializing
#include "Mp7JrsArchive.h"

// factory registry and defines
#include "Mp7JrsFactory.h"
#include "Mp7JrsFactoryDefines.h"
#endif


int main(int argc, char** argv) {

	try {
	
		// initializes Xerces
		XMLPlatformUtils::Initialize();
		// initialize MPEG-7 JRS
		Mp7JrsFactory::Initialize();

		// the following code goes into its own block to make sure 
		// that the smart pointer fall out of scope and are destructed
		// before the factory needed for destruction is deleted
		{
			// create archive instance
			Mp7JrsArchive archive;

			// set encoding and validation mode
			archive.SetEncoding("UTF-8");
			archive.SetDoValidation(false);

			Mp7JrsNodePtr node = CreateVideoSegmentType;

			// load and validate MPEG-7 XML file
			// returns the root node of the document
 			Mp7JrsNodePtr root = archive.FromFile("test.xml");

			// serialize the root node again
			archive.ToFile("test2.xml",root);

		}
		


		// cleanup MPEG-7 JRS
		Mp7JrsFactory::Terminate();
		// cleanup Xerces
		XMLPlatformUtils::Terminate();
	}
	catch (Mp7JrsException& e) 
	{
	
		printf("%s\n",e.GetMessage());
	}
	printf("done.");

	return 0;
}
